@@ -1,21 +1,21 @@
-/* gdaui-data-cell-renderer-pict.c
- *
- * Copyright (C) 2006 - 2010 Vivien Malerba <malerba@gdaui.org>
+/*
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <stdlib.h>
@@ -355,7 +355,7 @@ gdaui_data_cell_renderer_pict_set_property (GObject *object,
  * in the model, thus causing the check button to reflect the state of
  * the model.
  *
- * Return value: the new cell renderer
+ * Returns: the new cell renderer
  */
 GtkCellRenderer *
 gdaui_data_cell_renderer_pict_new (GdaDataHandler *dh, GType type, const gchar *options)
@@ -427,12 +427,13 @@ gdaui_data_cell_renderer_pict_render (GtkCellRenderer      *cell,
 }
 
 static void
-pict_data_changed_cb (GdauiDataCellRendererPict *pictcell)
+pict_data_changed_cb (PictBinData *bindata, GdauiDataCellRendererPict *pictcell)
 {
 	GValue *value;
 
-	value = common_pict_get_value (&(pictcell->priv->bindata), &(pictcell->priv->options),
+	value = common_pict_get_value (bindata, &(pictcell->priv->options),
 				       pictcell->priv->type);
+	g_free (bindata->data);
 	g_signal_emit (G_OBJECT (pictcell), pixbuf_cell_signals[CHANGED], 0,
 		       g_object_get_data (G_OBJECT (pictcell), "last-path"), value);
 	gda_value_free (value);
@@ -454,10 +455,13 @@ gdaui_data_cell_renderer_pict_activate  (GtkCellRenderer            *cell,
 		int event_time;
 
 		g_object_set_data_full (G_OBJECT (pictcell), "last-path", g_strdup (path), g_free);
-		if (!pictcell->priv->popup_menu.menu)
-			common_pict_create_menu (&(pictcell->priv->popup_menu), widget, &(pictcell->priv->bindata),
-						 &(pictcell->priv->options),
-						 (PictCallback) pict_data_changed_cb, pictcell);
+		if (pictcell->priv->popup_menu.menu) {
+			gtk_widget_destroy (pictcell->priv->popup_menu.menu);
+			pictcell->priv->popup_menu.menu = NULL;
+		}
+		common_pict_create_menu (&(pictcell->priv->popup_menu), widget, &(pictcell->priv->bindata),
+					 &(pictcell->priv->options),
+					 (PictCallback) pict_data_changed_cb, pictcell);
 
 		common_pict_adjust_menu_sensitiveness (&(pictcell->priv->popup_menu), pictcell->priv->editable,
 						       &(pictcell->priv->bindata));
