@@ -1,26 +1,27 @@
-/* GDA library
- * Copyright (C) 1998 - 2010 The GNOME Foundation.
+/*
+ * Copyright (C) 2005 Andrew Hill <andru@src.gnome.org>
+ * Copyright (C) 2005 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 lvaro Pea <alvaropg@telefonica.net>
+ * Copyright (C) 2007 - 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2008 Johannes Schmid <johannes.schmid@openismus.com>
+ * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
- * AUTHORS:
- *      Michael Lausch <michael@lausch.at>
- *	Rodrigo Moya <rodrigo@gnome-db.org>
- *	lvaro Pea <alvaropg@telefonica.net>
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <libgda/gda-column.h>
@@ -138,7 +139,7 @@ gda_column_class_init (GdaColumnClass *klass)
 }
 
 static void
-gda_column_init (GdaColumn *column, GdaColumnClass *klass)
+gda_column_init (GdaColumn *column, G_GNUC_UNUSED GdaColumnClass *klass)
 {
 	g_return_if_fail (GDA_IS_COLUMN (column));
 	
@@ -162,8 +163,7 @@ gda_column_finalize (GObject *object)
 	g_return_if_fail (GDA_IS_COLUMN (column));
 	
 	if (column->priv) {
-		if (column->priv->default_value) 
-			gda_value_free (column->priv->default_value);
+		gda_value_free (column->priv->default_value);
 	
 		g_free (column->priv->id);
 		g_free (column->priv->dbms_type);
@@ -191,7 +191,8 @@ gda_column_get_type (void)
 			NULL,
 			sizeof (GdaColumn),
 			0,
-			(GInstanceInitFunc) gda_column_init
+			(GInstanceInitFunc) gda_column_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -221,7 +222,8 @@ gda_column_set_property (GObject *object,
 				col->priv->id = NULL;
 			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -241,7 +243,8 @@ gda_column_get_property (GObject *object,
 			g_value_set_string (value, col->priv->id);
 			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -267,7 +270,7 @@ gda_column_new (void)
  * 	 
  * Creates a new #GdaColumn object from an existing one.
  *
- * Returns: a newly allocated #GdaColumn with a copy of the data 	 
+ * Returns: (transfer full): a newly allocated #GdaColumn with a copy of the data 	 
  * in @column. 	 
  */ 	 
 GdaColumn * 	 
@@ -543,7 +546,7 @@ gda_column_set_position (GdaColumn *column, gint position)
  * gda_column_get_default_value:
  * @column: a #GdaColumn.
  *
- * Returns: @column's default value, as a #GValue object.
+ * Returns: (allow-none): @column's default value, as a #GValue object, or %NULL if column does not have a default value
  */
 const GValue *
 gda_column_get_default_value (GdaColumn *column)
@@ -555,7 +558,7 @@ gda_column_get_default_value (GdaColumn *column)
 /**
  * gda_column_set_default_value:
  * @column: a #GdaColumn.
- * @default_value: default #GValue for the column
+ * @default_value: (allow-none): default #GValue for the column
  *
  * Sets @column's default #GValue.
  */
@@ -563,11 +566,12 @@ void
 gda_column_set_default_value (GdaColumn *column, const GValue *default_value)
 {
 	g_return_if_fail (GDA_IS_COLUMN (column));
-	g_return_if_fail (default_value != NULL);
 
-	if (column->priv->default_value)
-		gda_value_free (column->priv->default_value);
-	column->priv->default_value = gda_value_copy ( (GValue*)default_value);
+	gda_value_free (column->priv->default_value);
+	if (default_value)
+		column->priv->default_value = gda_value_copy ( (GValue*)default_value);
+	else
+		column->priv->default_value = NULL;
 }
 
 /**
@@ -592,8 +596,8 @@ gda_column_get_attribute (GdaColumn *column, const gchar *attribute)
  * gda_column_set_attribute:
  * @column: a #GdaColumn
  * @attribute: attribute name as a static string
- * @value: a #GValue, or %NULL
- * @destroy: a function to be called when @attribute is not needed anymore, or %NULL
+ * @value: (allow-none): a #GValue, or %NULL
+ * @destroy: (allow-none): a function to be called when @attribute is not needed anymore, or %NULL
  *
  * Set the value associated to a named attribute. The @attribute string is 'stolen' by this method, and
  * the memory it uses will be freed using the @destroy function when no longer needed (if @destroy is %NULL,
