@@ -1,20 +1,31 @@
 /*
- * Copyright (C) 1998 - 2008 The GNOME Foundation.
+ * Copyright (C) 2000 Reinhard Müller <reinhard@src.gnome.org>
+ * Copyright (C) 2000 - 2002 Rodrigo Moya <rodrigo@gnome-db.org>
+ * Copyright (C) 2001 Carlos Perell Marn <carlos@gnome-db.org>
+ * Copyright (C) 2001 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2002 Andrew Hill <andru@src.gnome.org>
+ * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ * Copyright (C) 2002 Xabier Rodriguez Calvar <xrcalvar@igalia.com>
+ * Copyright (C) 2002 Zbigniew Chyla <cyba@gnome.pl>
+ * Copyright (C) 2003 Laurent Sansonetti <laurent@datarescue.be>
+ * Copyright (C) 2006 - 2007 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
+ * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <glib.h>
@@ -54,7 +65,7 @@ static gboolean lang_locale_dyn = FALSE;
 gchar          *gda_lang_locale = "";
 
 /**
- * gda_locale_changed
+ * gda_locale_changed:
  *
  * Call this function whenever the setlocale() function has been called
  * to change the current locale; this function is first called by gda_init() so you
@@ -100,7 +111,7 @@ gda_locale_changed (void)
 }
 
 /**
- * gda_init
+ * gda_init:
  * 
  * Initializes the GDA library, must be called prior to any Libgda usage. Note that unless the
  * LIBGDA_NO_THREADS environment variable is set (to any value), the GLib thread system will
@@ -171,6 +182,9 @@ gda_init (void)
 	type = GDA_TYPE_ERROR;
 	g_assert (type);
 
+	/* force TZ init */
+	tzset ();
+
 	/* acquire locale */
 	gda_locale_changed ();
 
@@ -246,7 +260,7 @@ gda_init (void)
 }
 
 /**
- * gda_get_application_exec_path
+ * gda_get_application_exec_path:
  * @app_name: the name of the application to find
  *
  * Find the path to the application identified by @app_name. For example if the application
@@ -254,7 +268,7 @@ gda_init (void)
  * "/your/prefix/bin/gda-sql-4.0" if Libgda is installed in
  * the "/your/prefix" prefix (which would usually be "/usr"), and for the ABI version 4.0.
  *
- * Returns: the path as a new string, or %NULL if the application cannot be found
+ * Returns: (transfer full): the path as a new string, or %NULL if the application cannot be found
  */
 gchar *
 gda_get_application_exec_path (const gchar *app_name)
