@@ -1,24 +1,21 @@
-/* 
- * GDA common library
- * Copyright (C) 2008 The GNOME Foundation.
+/*
+ * Copyright (C) 2008 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <libgda/gda-lockable.h>
@@ -41,7 +38,8 @@ gda_lockable_get_type (void)
 			NULL,
 			0,
 			0,
-			(GInstanceInitFunc) NULL
+			(GInstanceInitFunc) NULL,
+			0
 		};
 		
 		g_static_rec_mutex_lock (&init_mutex);
@@ -55,7 +53,7 @@ gda_lockable_get_type (void)
 }
 
 static void
-gda_lockable_class_init (gpointer g_class)
+gda_lockable_class_init (G_GNUC_UNUSED gpointer g_class)
 {
 	static gboolean initialized = FALSE;
 
@@ -67,7 +65,7 @@ gda_lockable_class_init (gpointer g_class)
 }
 
 /**
- * gda_lockable_lock
+ * gda_lockable_lock:
  * @lockable: a #GdaLockable object.
  *
  * Locks @lockable. If it is already locked by another thread, the current thread will block until it is unlocked 
@@ -90,7 +88,7 @@ gda_lockable_lock (GdaLockable *lockable)
 }
 
 /**
- * gda_lockable_trylock
+ * gda_lockable_trylock:
  * @lockable: a #GdaLockable object.
  *
  * Tries to lock @lockable. If it is already locked by another thread, then it immediately returns FALSE, otherwise
@@ -117,7 +115,7 @@ gda_lockable_trylock (GdaLockable *lockable)
 }
 
 /**
- * gda_lockable_unlock
+ * gda_lockable_unlock:
  * @lockable: a #GdaLockable object.
  *
  * Unlocks @lockable. This method should not be called if the current does not already holds a lock on @lockable (having
