@@ -112,6 +112,13 @@ gda_g_type_to_string (GType type)
  * some type synonyms such as:
  * <itemizedlist>
  *   <listitem><para>"int" for G_TYPE_INT</para></listitem>
+ *   <listitem><para>"uint" for G_TYPE_UINT</para></listitem>
+ *   <listitem><para>"int64" for G_TYPE_INT64</para></listitem>
+ *   <listitem><para>"uint64" for G_TYPE_UINT64</para></listitem>
+ *   <listitem><para>"char" for G_TYPE_CHAR</para></listitem>
+ *   <listitem><para>"uchar" for G_TYPE_UCHAR</para></listitem>
+ *   <listitem><para>"short" for GDA_TYPE_SHORT</para></listitem>
+ *   <listitem><para>"ushort" for GDA_TYPE_USHORT</para></listitem>
  *   <listitem><para>"string" for G_TYPE_STRING</para></listitem>
  *   <listitem><para>"date" for G_TYPE_DATE</para></listitem>
  *   <listitem><para>"time" for GDA_TYPE_TIME</para></listitem>
@@ -134,6 +141,8 @@ gda_g_type_from_string (const gchar *str)
 	if (type == 0) {
 		if (!g_ascii_strcasecmp (str, "int"))
 			type = G_TYPE_INT;
+		else if (!g_ascii_strcasecmp (str, "uint"))
+			type = G_TYPE_UINT;
 		else if (!g_ascii_strcasecmp (str, "string"))
 			type = G_TYPE_STRING;
 		else if (!g_ascii_strcasecmp (str, "date"))
@@ -150,6 +159,18 @@ gda_g_type_from_string (const gchar *str)
 			type = GDA_TYPE_BINARY;
 		else if (!strcmp (str, "null"))
 			type = GDA_TYPE_NULL;
+		else if (!strcmp (str, "short"))
+			type = GDA_TYPE_SHORT;
+		else if (!strcmp (str, "ushort"))
+			type = GDA_TYPE_USHORT;
+		else if (!g_ascii_strcasecmp (str, "int64"))
+			type = G_TYPE_INT64;
+		else if (!g_ascii_strcasecmp (str, "uint64"))
+			type = G_TYPE_UINT64;
+		else if (!g_ascii_strcasecmp (str, "char"))
+			type = G_TYPE_CHAR;
+		else if (!g_ascii_strcasecmp (str, "uchar"))
+			type = G_TYPE_UCHAR;
 		else
 			/* could not find a valid GType for @str */
 			type = G_TYPE_INVALID;
@@ -594,7 +615,7 @@ gda_utility_data_model_find_column_description (GdaDataSelect *model, const gcha
  * gda_utility_holder_load_attributes:
  * @holder: a #GdaHolder
  * @node: an xmlNodePtr with a &lt;parameter&gt; tag
- * @sources: a list of #GdaDataModel
+ * @sources: (element-type Gda.DataModel): a list of #GdaDataModel
  * @error: a place to store errors, or %NULL
  *
  * Note: this method may set the "source" custom string property
@@ -890,17 +911,19 @@ dml_statements_check_select_structure (GdaConnection *cnc, GdaSqlStatement *sel_
 }
 
 /**
- * gda_compute_unique_table_row_condition_with_cnc:
+ * gda_compute_unique_table_row_condition_with_cnc: (skip)
  * @cnc: (allow-none): a #GdaConnection, or %NULL
  * @stsel: a #GdaSqlSelectStatement
  * @mtable: a #GdaMetaTable
- * @require_pk: set to TRUE if a primary key ir required
+ * @require_pk: set to %TRUE if a primary key is required
  * @error: a place to store errors, or %NULL
  * 
  * Computes a #GdaSqlExpr expression which can be used in the WHERE clause of an UPDATE
  * or DELETE statement when a row from the result of the @stsel statement has to be modified.
  *
- * REM: the case where @require_pk is %FALSE is not currently implemented.
+ * If @require_pk is %TRUE then this function will return a non %NULL #GdaSqlExpr only if it can
+ * use a primary key of @mtable. If @require_pk is %FALSE, then it will try to use a primary key of @mtable,
+ * and if none is available, it will use all the columns of @mtable to compute a condition statement.
  *
  * Returns: a new #GdaSqlExpr, or %NULL if an error occurred.
  *
@@ -917,147 +940,152 @@ gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc, GdaSqlState
 	g_return_val_if_fail (!cnc || GDA_IS_CONNECTION (cnc), NULL);
 	
 	expr = gda_sql_expr_new (NULL); /* no parent */
-	if (require_pk) {
-		if (mtable->pk_cols_nb == 0) {
-			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-				     "%s", _("Table does not have any primary key"));
-			goto onerror;
-		}
-		if (mtable->pk_cols_nb == 0) {
-			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-					     "%s", _("Table does not have any primary key"));
-			goto onerror;
-		}
-		else if (mtable->pk_cols_nb > 1) {
-			and_cond = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
-			and_cond->operator_type = GDA_SQL_OPERATOR_TYPE_AND;
-			expr->cond = and_cond;
-		}
-		for (i = 0; i < mtable->pk_cols_nb; i++) {
-			GdaSqlSelectField *sfield = NULL;
-			GdaMetaTableColumn *tcol;
-			GSList *list;
-			gint index;
 
-			tcol = (GdaMetaTableColumn *) g_slist_nth_data (mtable->columns, mtable->pk_cols_array[i]);
-			for (index = 0, list = stsel->expr_list;
-			     list;
-			     index++, list = list->next) {
-				sfield = (GdaSqlSelectField *) list->data;
-				if (sfield->validity_meta_table_column == tcol)
-					break;
-				else
-					sfield = NULL;
-			}
-			if (!sfield) {
-				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-					     "%s", _("Table's primary key is not part of SELECT"));
+	if (mtable->pk_cols_nb == 0) {
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+			     "%s", _("Table does not have any primary key"));
+		if (require_pk)
+			goto onerror;
+		else
+			goto allcolums;
+	}
+	else if (mtable->pk_cols_nb > 1) {
+		and_cond = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
+		and_cond->operator_type = GDA_SQL_OPERATOR_TYPE_AND;
+		expr->cond = and_cond;
+	}
+	for (i = 0; i < mtable->pk_cols_nb; i++) {
+		GdaSqlSelectField *sfield = NULL;
+		GdaMetaTableColumn *tcol;
+		GSList *list;
+		gint index;
+		
+		tcol = (GdaMetaTableColumn *) g_slist_nth_data (mtable->columns, mtable->pk_cols_array[i]);
+		for (index = 0, list = stsel->expr_list;
+		     list;
+		     index++, list = list->next) {
+			sfield = (GdaSqlSelectField *) list->data;
+			if (sfield->validity_meta_table_column == tcol)
+				break;
+			else
+				sfield = NULL;
+		}
+		if (!sfield) {
+			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+				     "%s", _("Table's primary key is not part of SELECT"));
+			if (require_pk)
 				goto onerror;
+			else
+				goto allcolums;
+		}
+		else {
+			GdaSqlOperation *op;
+			GdaSqlExpr *opexpr;
+			GdaSqlParamSpec *pspec;
+			
+			/* equal condition */
+			if (and_cond) {
+				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (and_cond));
+				op = gda_sql_operation_new (GDA_SQL_ANY_PART (opexpr));
+				opexpr->cond = op;
+				and_cond->operands = g_slist_append (and_cond->operands, opexpr);
 			}
 			else {
-				GdaSqlOperation *op;
-				GdaSqlExpr *opexpr;
-				GdaSqlParamSpec *pspec;
-
-				/* equal condition */
-				if (and_cond) {
-					opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (and_cond));
-					op = gda_sql_operation_new (GDA_SQL_ANY_PART (opexpr));
-					opexpr->cond = op;
-					and_cond->operands = g_slist_append (and_cond->operands, opexpr);
-				}
-				else {
-					op = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
-					expr->cond = op;
-				}
-				op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
-				/* left operand */
-				gchar *str;
-				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
-				str = gda_sql_identifier_quote (tcol->column_name, cnc, NULL, FALSE, FALSE);
-				g_value_take_string (opexpr->value = gda_value_new (G_TYPE_STRING), str);
-
-				op->operands = g_slist_append (op->operands, opexpr);
-
-				/* right operand */
-				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
-				pspec = g_new0 (GdaSqlParamSpec, 1);
-				pspec->name = g_strdup_printf ("-%d", index);
-				pspec->g_type = tcol->gtype != G_TYPE_INVALID ? tcol->gtype: G_TYPE_STRING;
-				pspec->nullok = tcol->nullok;
-				opexpr->param_spec = pspec;
-				op->operands = g_slist_append (op->operands, opexpr);
+				op = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
+				expr->cond = op;
 			}
+			op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
+			/* left operand */
+			gchar *str;
+			opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+			str = gda_sql_identifier_quote (tcol->column_name, cnc, NULL, FALSE, FALSE);
+			g_value_take_string (opexpr->value = gda_value_new (G_TYPE_STRING), str);
+			
+			op->operands = g_slist_append (op->operands, opexpr);
+			
+			/* right operand */
+			opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+			pspec = g_new0 (GdaSqlParamSpec, 1);
+			pspec->name = g_strdup_printf ("-%d", index);
+			pspec->g_type = (tcol->gtype != GDA_TYPE_NULL) ? tcol->gtype: G_TYPE_STRING;
+			pspec->nullok = tcol->nullok;
+			opexpr->param_spec = pspec;
+			op->operands = g_slist_append (op->operands, opexpr);
 		}
 	}
-	else {
-		GSList *columns;
-		if (! mtable->columns) {
+	return expr;
+
+ allcolums:
+
+	gda_sql_expr_free (expr);
+	expr = gda_sql_expr_new (NULL); /* no parent */
+
+	GSList *columns;
+	if (! mtable->columns) {
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+			     "%s", _("Table does not have any column"));
+		goto onerror;
+	}
+	else if (mtable->columns->next) {
+		and_cond = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
+		and_cond->operator_type = GDA_SQL_OPERATOR_TYPE_AND;
+		expr->cond = and_cond;
+	}
+	for (columns = mtable->columns; columns; columns = columns->next) {
+		GdaSqlSelectField *sfield = NULL;
+		GdaMetaTableColumn *tcol;
+		GSList *list;
+		gint index;
+		
+		tcol = (GdaMetaTableColumn *) columns->data;
+		for (index = 0, list = stsel->expr_list;
+		     list;
+		     index++, list = list->next) {
+			sfield = (GdaSqlSelectField *) list->data;
+			if (sfield->validity_meta_table_column == tcol)
+				break;
+			else
+				sfield = NULL;
+		}
+		if (!sfield) {
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-					     "%s", _("Table does not have any column"));
+				     _("Table's column '%s' is not part of SELECT"),
+				     tcol->column_name);
 			goto onerror;
 		}
-		else if (mtable->columns->next) {
-			and_cond = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
-			and_cond->operator_type = GDA_SQL_OPERATOR_TYPE_AND;
-			expr->cond = and_cond;
-		}
-		for (columns = mtable->columns; columns; columns = columns->next) {
-			GdaSqlSelectField *sfield = NULL;
-			GdaMetaTableColumn *tcol;
-			GSList *list;
-			gint index;
-
-			tcol = (GdaMetaTableColumn *) columns->data;
-			for (index = 0, list = stsel->expr_list;
-			     list;
-			     index++, list = list->next) {
-				sfield = (GdaSqlSelectField *) list->data;
-				if (sfield->validity_meta_table_column == tcol)
-					break;
-				else
-					sfield = NULL;
-			}
-			if (!sfield) {
-				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-					     _("Table's column '%s' is not part of SELECT"),
-					     tcol->column_name);
-				goto onerror;
+		else {
+			GdaSqlOperation *op;
+			GdaSqlExpr *opexpr;
+			GdaSqlParamSpec *pspec;
+			
+			/* equal condition */
+			if (and_cond) {
+				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (and_cond));
+				op = gda_sql_operation_new (GDA_SQL_ANY_PART (opexpr));
+				opexpr->cond = op;
+				and_cond->operands = g_slist_append (and_cond->operands, opexpr);
 			}
 			else {
-				GdaSqlOperation *op;
-				GdaSqlExpr *opexpr;
-				GdaSqlParamSpec *pspec;
-
-				/* equal condition */
-				if (and_cond) {
-					opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (and_cond));
-					op = gda_sql_operation_new (GDA_SQL_ANY_PART (opexpr));
-					opexpr->cond = op;
-					and_cond->operands = g_slist_append (and_cond->operands, opexpr);
-				}
-				else {
-					op = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
-					expr->cond = op;
-				}
-				op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
-				/* left operand */
-				gchar *str;
-				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
-				str = gda_sql_identifier_quote (tcol->column_name, cnc, NULL, FALSE, FALSE);
-				g_value_take_string (opexpr->value = gda_value_new (G_TYPE_STRING), str);
-
-				op->operands = g_slist_append (op->operands, opexpr);
-
-				/* right operand */
-				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
-				pspec = g_new0 (GdaSqlParamSpec, 1);
-				pspec->name = g_strdup_printf ("-%d", index);
-				pspec->g_type = (tcol->gtype != GDA_TYPE_NULL) ? tcol->gtype: G_TYPE_STRING;
-				pspec->nullok = tcol->nullok;
-				opexpr->param_spec = pspec;
-				op->operands = g_slist_append (op->operands, opexpr);
+				op = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
+				expr->cond = op;
 			}
+			op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
+			/* left operand */
+			gchar *str;
+			opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+			str = gda_sql_identifier_quote (tcol->column_name, cnc, NULL, FALSE, FALSE);
+			g_value_take_string (opexpr->value = gda_value_new (G_TYPE_STRING), str);
+			
+			op->operands = g_slist_append (op->operands, opexpr);
+			
+			/* right operand */
+			opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+			pspec = g_new0 (GdaSqlParamSpec, 1);
+			pspec->name = g_strdup_printf ("-%d", index);
+			pspec->g_type = (tcol->gtype != GDA_TYPE_NULL) ? tcol->gtype: G_TYPE_STRING;
+			pspec->nullok = tcol->nullok;
+			opexpr->param_spec = pspec;
+			op->operands = g_slist_append (op->operands, opexpr);
 		}
 	}
 	return expr;
@@ -1068,7 +1096,7 @@ gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc, GdaSqlState
 }
 
 /**
- * gda_compute_unique_table_row_condition:
+ * gda_compute_unique_table_row_condition: (skip)
  * @stsel: a #GdaSqlSelectStatement
  * @mtable: a #GdaMetaTable
  * @require_pk: set to TRUE if a primary key ir required
@@ -1309,7 +1337,7 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 }
 
 /**
- * gda_compute_select_statement_from_update:
+ * gda_compute_select_statement_from_update: (skip)
  * @update_stmt: an UPDATE statement
  * @error: a place to store errors, or %NULL
  *
@@ -1643,7 +1671,7 @@ static gboolean stmt_rewrite_update_default_keyword (GdaSqlStatementUpdate *upd,
 
 
 /**
- * gda_statement_rewrite_for_default_values:
+ * gda_statement_rewrite_for_default_values: (skip)
  * @stmt: a #GdaStatement object
  * @params: a #GdaSet containing the variable's values to be bound when executing @stmt
  * @remove: set to %TRUE if DEFAULT fields are removed, of %FALSE if the "DEFAULT" keyword is used
