@@ -1,23 +1,33 @@
-/* GDA Postgres provider
- * Copyright (C) 2008 The GNOME Foundation.
+/*
+ * Copyright (C) 2001 - 2003 Gonzalo Paniagua Javier <gonzalo@ximian.com>
+ * Copyright (C) 2001 - 2004 Rodrigo Moya <rodrigo@ximian.com>
+ * Copyright (C) 2002 Zbigniew Chyla <cyba@gnome.pl>
+ * Copyright (C) 2003 Akira TAGOH <tagoh@gnome-db.org>
+ * Copyright (C) 2003 Laurent Sansonetti <laurent@datarescue.be>
+ * Copyright (C) 2004 Andrew Hill <andru@src.gnome.org>
+ * Copyright (C) 2004 - 2005 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2004 Szalai Ferenc <szferi@einstein.ki.iif.hu>
+ * Copyright (C) 2004 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 Alex <alex@igalia.com>
+ * Copyright (C) 2005 lvaro Pea <alvaropg@telefonica.net>
+ * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <stdarg.h>
@@ -30,6 +40,7 @@
 #include "gda-postgres-provider.h"
 #include "gda-postgres-blob-op.h"
 #include "gda-postgres-util.h"
+#include <libgda/libgda-global-variables.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -363,7 +374,7 @@ gda_postgres_recordset_new_cursor (GdaConnection *cnc, GdaPostgresPStmt *ps, Gda
 	pg_res = PQexec (cdata->pconn, str);
 	g_free (str);
 	status = PQresultStatus (pg_res);
-	if (!pg_res || (PQresultStatus (pg_res) != PGRES_TUPLES_OK)) {
+	if (!pg_res || (status != PGRES_TUPLES_OK)) {
 		_gda_postgres_make_error (cdata->cnc, cdata->pconn, pg_res, NULL);
 		if (pg_res) {
 			PQclear (pg_res);
@@ -426,13 +437,10 @@ gda_postgres_recordset_fetch_random (GdaDataSelect *model, GdaRow **prow, gint r
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset *) model;
 
-	if (*prow)
-		return TRUE;
-
 	if (!imodel->priv->pg_res) {
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_INTERNAL_ERROR,
 			     "%s", _("Internal error"));
-		return FALSE;
+		return TRUE;
 	}
 
 	*prow = new_row_from_pg_res (imodel, rownum, error);
@@ -475,23 +483,19 @@ gda_postgres_recordset_store_all (GdaDataSelect *model, GError **error)
  * Create a new filled #GdaRow object for the next cursor row, and put it into *prow.
  *
  * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
- * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
+ * never keeps a reference to it).
  */
 static gboolean
 gda_postgres_recordset_fetch_next (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
-	if (*prow)
-		return TRUE;
-
 	if (row_is_in_current_pg_res (imodel, rownum)) {
 		if (imodel->priv->tmp_row)
 			set_prow_with_pg_res (imodel, imodel->priv->tmp_row, rownum - imodel->priv->pg_res_inf, error);
 		else
 			imodel->priv->tmp_row = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 		*prow = imodel->priv->tmp_row;
-		return TRUE;
 	}
 	else {
 		gboolean fetch_error = FALSE;
@@ -501,34 +505,28 @@ gda_postgres_recordset_fetch_next (GdaDataSelect *model, GdaRow **prow, gint row
 			else
 				imodel->priv->tmp_row = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 			*prow = imodel->priv->tmp_row;
-			return TRUE;
 		}
-		else
-			return !fetch_error;
 	}
+	return TRUE;
 }
 
 /*
  * Create a new filled #GdaRow object for the previous cursor row, and put it into *prow.
  *
  * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
- * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
+ * never keeps a reference to it).
  */
 static gboolean
 gda_postgres_recordset_fetch_prev (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
-	if (*prow)
-		return TRUE;
-
 	if (row_is_in_current_pg_res (imodel, rownum)) {
 		if (imodel->priv->tmp_row)
 			set_prow_with_pg_res (imodel, imodel->priv->tmp_row, rownum - imodel->priv->pg_res_inf, error);
 		else
 			imodel->priv->tmp_row = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 		*prow = imodel->priv->tmp_row;
-		return TRUE;
 	}
 	else {
 		gboolean fetch_error = FALSE;
@@ -538,27 +536,22 @@ gda_postgres_recordset_fetch_prev (GdaDataSelect *model, GdaRow **prow, gint row
 			else
 				imodel->priv->tmp_row = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 			*prow = imodel->priv->tmp_row;
-			return TRUE;
 		}
-		else
-			return !fetch_error;
 	}
+	return TRUE;
 }
 
 /*
  * Create a new filled #GdaRow object for the cursor row at position @rownum, and put it into *prow.
  *
  * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
- * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
+ * never keeps a reference to it).
  */
 static gboolean
 gda_postgres_recordset_fetch_at (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
-	if (*prow)
-		return TRUE;
-
 	if (imodel->priv->tmp_row) {
 		g_object_unref (imodel->priv->tmp_row);
 		imodel->priv->tmp_row = NULL;
@@ -567,18 +560,15 @@ gda_postgres_recordset_fetch_at (GdaDataSelect *model, GdaRow **prow, gint rownu
 	if (row_is_in_current_pg_res (imodel, rownum)) {
 		*prow = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 		imodel->priv->tmp_row = *prow;
-		return TRUE;
 	}
 	else {
 		gboolean fetch_error = FALSE;
 		if (fetch_row_number_chunk (imodel, rownum, &fetch_error, error)) {
 			*prow = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 			imodel->priv->tmp_row = *prow;
-			return TRUE;
 		}
-		else
-			return !fetch_error;
 	}
+	return TRUE;
 }
 
 
@@ -646,16 +636,14 @@ set_value (GdaConnection *cnc, GdaRow *row, GValue *value, GType type, const gch
 	else if (type == GDA_TYPE_SHORT)
 		gda_value_set_short (value, atoi (thevalue));
 	else if (type == G_TYPE_FLOAT) {
-		char *current_locale;
-		current_locale = setlocale (LC_NUMERIC, "C");
+		setlocale (LC_NUMERIC, "C");
 		g_value_set_float (value, atof (thevalue));
-		setlocale (LC_NUMERIC, current_locale);
+		setlocale (LC_NUMERIC, gda_numeric_locale);
 	}
 	else if (type == G_TYPE_DOUBLE) {
-		char *current_locale;
-		current_locale = setlocale (LC_NUMERIC, "C");
+		setlocale (LC_NUMERIC, "C");
 		g_value_set_double (value, atof (thevalue));
-		setlocale (LC_NUMERIC, current_locale);
+		setlocale (LC_NUMERIC, gda_numeric_locale);
 	}
 	else if (type == GDA_TYPE_NUMERIC) {
 		GdaNumeric numeric;
