@@ -521,7 +521,8 @@ gda_capi_provider_create_operation (GdaServerProvider *provider, GdaConnection *
 	g_free (dir);
 
         if (! file) {
-                g_set_error (error, 0, 0, _("Missing spec. file '%s'"), str);
+                g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_FILE_NOT_FOUND_ERROR,
+			     _("Missing spec. file '%s'"), str);
 		g_free (str);
                 return NULL;
         }
@@ -560,7 +561,8 @@ gda_capi_provider_render_operation (GdaServerProvider *provider, GdaConnection *
 	g_free (dir);
 
         if (! file) {
-                g_set_error (error, 0, 0, _("Missing spec. file '%s'"), str);
+                g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_FILE_NOT_FOUND_ERROR,
+			     _("Missing spec. file '%s'"), str);
 		g_free (str);
                 return NULL;
         }
@@ -1179,7 +1181,33 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			else if (!rstmt)
 				return NULL;
 			else {
+				/* The strategy here is to execute @rstmt using its prepared
+				 * statement, but with common data from @ps. Beware that
+				 * the @param_ids attribute needs to be retained (i.e. it must not
+				 * be the one copied from @ps) */
 				GObject *obj;
+				GdaCapiPStmt *tps;
+				GdaPStmt *gtps;
+				GSList *prep_param_ids, *copied_param_ids;
+				if (!gda_capi_provider_statement_prepare (provider, cnc,
+									  rstmt, error))
+					return NULL;
+				tps = (GdaCapiPStmt *)
+					gda_connection_get_prepared_statement (cnc, rstmt);
+				gtps = (GdaPStmt *) tps;
+
+				/* keep @param_ids to avoid being cleared by gda_pstmt_copy_contents() */
+				prep_param_ids = gtps->param_ids;
+				gtps->param_ids = NULL;
+				
+				/* actual copy */
+				gda_pstmt_copy_contents ((GdaPStmt *) ps, (GdaPStmt *) tps);
+
+				/* restore previous @param_ids */
+				copied_param_ids = gtps->param_ids;
+				gtps->param_ids = prep_param_ids;
+
+				/* execute */
 				obj = gda_capi_provider_statement_execute (provider, cnc,
 									   rstmt, params,
 									   model_usage,
@@ -1187,15 +1215,16 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 									   last_inserted_row,
 									   task_id, async_cb,
 									   cb_data, error);
+				/* clear original @param_ids and restore copied one */
+				g_slist_foreach (prep_param_ids, (GFunc) g_free, NULL);
+				g_slist_free (prep_param_ids);
+
+				gtps->param_ids = copied_param_ids;
+
+				/*if (GDA_IS_DATA_MODEL (obj))
+				  gda_data_model_dump ((GdaDataModel*) obj, NULL);*/
+
 				g_object_unref (rstmt);
-				if (GDA_IS_DATA_SELECT (obj)) {
-					GdaPStmt *pstmt;
-					g_object_get (obj, "prepared-stmt", &pstmt, NULL);
-					if (pstmt) {
-						gda_pstmt_set_gda_statement (pstmt, stmt);
-						g_object_unref (pstmt);
-					}
-				}
 				return obj;
 			}
 		}
