@@ -1,21 +1,20 @@
-/* browser-canvas-print.c
+/*
+ * Copyright (C) 2009 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * Copyright (C) 2007 Vivien Malerba
- *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
+ * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -100,20 +99,21 @@ browser_canvas_print (BrowserCanvas *canvas)
 }
 
 static void
-print_begin (GtkPrintOperation *operation, GtkPrintContext *context, PrintPageData *pdata)
+print_begin (GtkPrintOperation *operation, G_GNUC_UNUSED GtkPrintContext *context, PrintPageData *pdata)
 {
 	gtk_print_operation_set_n_pages (operation, pdata->h_npages * pdata->v_npages);
 	gtk_print_operation_set_default_page_setup (operation, pdata->page_setup);
 }
 
 static void
-print_end (GtkPrintOperation *operation, GtkPrintContext *context, PrintPageData *pdata)
+print_end (G_GNUC_UNUSED GtkPrintOperation *operation, G_GNUC_UNUSED GtkPrintContext *context,
+	   G_GNUC_UNUSED PrintPageData *pdata)
 {
 	
 }
 
 static void
-print_draw_page (GtkPrintOperation *operation, GtkPrintContext *context, gint page_nr, PrintPageData *pdata)
+print_draw_page (G_GNUC_UNUSED GtkPrintOperation *operation, GtkPrintContext *context, gint page_nr, PrintPageData *pdata)
 {
 	cairo_t *cr;
 	GooCanvasBounds bounds, canvas_bounds;
@@ -218,7 +218,7 @@ static void print_zoom_value_changed_cb (GtkSpinButton *entry, PrintCustomData *
 static void print_page_numbers_toggled_cb (GtkToggleButton *toggle, PrintCustomData *cdata);
 
 static GObject *
-print_create_custom_widget_cb (GtkPrintOperation *operation, PrintPageData *pdata)
+print_create_custom_widget_cb (G_GNUC_UNUSED GtkPrintOperation *operation, PrintPageData *pdata)
 {
 	GtkWidget *vbox, *bbox, *button, *label, *hbox, *table, *entry;
 	PrintCustomData *cdata;
