@@ -1,25 +1,20 @@
-/* GNOME DB library
+/*
+ * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * Copyright (C) 1999 - 2009 The GNOME Foundation.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * AUTHORS:
- *      Rodrigo Moya <rodrigo@gnome-db.org>
- *      Vivien Malerba <malerba@gnome-db.org>
- *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <gtk/gtk.h>
@@ -309,7 +304,7 @@ cc_gray_bar_class_init (CcGrayBarClass *klass)
 }
 
 static void
-cc_gray_bar_init (CcGrayBar *bar, CcGrayBarClass *klass)
+cc_gray_bar_init (CcGrayBar *bar, G_GNUC_UNUSED CcGrayBarClass *klass)
 {
 #if GTK_CHECK_VERSION(2,18,0)
 	gtk_widget_set_has_window (GTK_WIDGET (bar), TRUE);
@@ -420,7 +415,8 @@ cc_gray_bar_get_type (void)
 			NULL,
 			sizeof (CcGrayBar),
 			0,
-			(GInstanceInitFunc) cc_gray_bar_init
+			(GInstanceInitFunc) cc_gray_bar_init,
+			0
 		};
 		type = g_type_register_static (GTK_TYPE_BIN, "CcGrayBar", &info, 0);
 	}
